/* Distributed Checksum Clearinghouse
 *
 * checksum routines
 *
 * Copyright (c) 2003 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.7-1.50 $Revision$
 */

#ifndef DCC_CK_H
#define DCC_CK_H

#include "dcc_clnt.h"
#include "dcc_md5.h"


/* relative fuzziness of checksums
 *	Should this be configurable? */
#define DCC_CK_FUZ_LVL_NO   1		/* least fuzzy */
#define DCC_CK_FUZ_LVL1	    2		/* somewhat fuzzy */
#define DCC_CK_FUZ_LVL2	    3		/* fuzzier */
extern const u_char dcc_ck_fuzziness[DCC_DIM_CKS];

/* MIME boundary */
/* RFC 1341 says boundaries can be 70 bytes, but handle non-conformant spam */
#define DCC_CK_BND_MAX	    94
#define DCC_CK_BND_DIM	(DCC_CK_BND_MAX+2)  /* including leading -- */
#define DCC_CK_BND_MISS	(DCC_CK_BND_DIM+1)
typedef struct {
    u_char	bnd_len;		/* length including leading "--" */
    u_char	cmp_len;		/* compared so far */
    char	bnd[DCC_CK_BND_DIM];	/* "--"boundary */
} DCC_CK_BND;


/* state machine for ignoring parts of URLs */
typedef struct {
    enum {
	DCC_URL_ST_IDLE,		/* waiting for H or = */
	DCC_URL_ST_QUOTE,		/*    "     " '"' or 'H' after = */
	DCC_URL_ST_QH,			/*    "     "  H after '"' */
	DCC_URL_ST_T1,			/*    "     "  T */
	DCC_URL_ST_T2,			/*    "     "  T */
	DCC_URL_ST_P,			/*    "     "  P */
	DCC_URL_ST_S,			/*    "     "  [S] */
	DCC_URL_ST_COLON,		/*    "     "  : */
	DCC_URL_ST_SLASH1,		/*    "     "  / */
	DCC_URL_ST_SLASH2,		/*    "     "  / */
	DCC_URL_ST_SLASH3_START,
	DCC_URL_ST_SLASH3,		/*    "     "  third / */
	DCC_URL_ST_SKIP,		/* skipping rest of URL */
    } st;
    char	*start;			/* start of hostname in URL */
    char	*dot;			/* last '.' in hostname */
    int		total;			/* length of URL */
    u_char	flags;
#    define	 DCC_URL_QUOTED	    0x01
#    define	 DCC_URL_DEL_DOMAIN 0x02    /* waiting for domain to delete */
#    define	 DCC_URL_PERCENT1   0x04    /* waiting for 1st digit after % */
#    define	 DCC_URL_PERCENT2   0x08    /* waiting for 2nd digit after % */
#    define	 DCC_URL_SIMPLE (DCC_URL_DEL_DOMAIN	\
				 | DCC_URL_PERCENT1	\
				 | DCC_URL_PERCENT2)
    u_char	percent;
} DCC_URL_SKIP;
#define DCC_URL_MAX	    65		/* 63 is RFC 1034 limit on labels */
#define DCC_URL_FAILSAFE    2000	/* big bogus "username:password@" */


/* accumulated checksums for an SMTP message */
typedef struct {
    const char *hdr;			/* name if substitute checksum */
    DCC_TGTS	tgts;			/* server's answer or previous total */
    DCC_CK_TYPE	type;
    u_char	rpt;			/* 1=report to server */
    DCC_SUM	sum;
} DCC_GOT_SUM;

typedef union {
    u_int32_t	w32[4];
    u_char	b[16];
} DCC_FUZ2_WORD;
#define DCC_FUZ2_WORD_CLEAR(_w) ((_w)->w32[0]=0, (_w)->w32[1]=0,	\
				 (_w)->w32[2]=0, (_w)->w32[3]=0)
typedef struct {
    DCC_URL_SKIP url;
    u_int	total;			/* bytes */
    char	*cp;			/* end of data in buffer */
    char	*eol;			/* most recent eol */
#    define	 DCC_FUZ1_MAX_LINE  78
#    define	 DCC_HTTPS_STR    "https"
#    define	 DCC_HTTPS_LEN	(sizeof(DCC_HTTPS_STR)-1)
    char	buf[DCC_FUZ1_MAX_LINE*4
		    +DCC_HTTPS_LEN];
    MD5_CTX	md5;
} DCC_CTX_FUZ1;

typedef struct {
    u_int	btotal;			/* total non-whitespace bytes seen */
    u_int	summed;			/* bytes summed */
    u_int	wtotal;			/* words summed */
    u_int	wlen;			/* length of current word */
    u_int	clen;			/* fancy character length */
    u_int	tlen;			/* HTML tag length */
    DCC_FUZ2_WORD w;
    DCC_FUZ2_WORD c;
    int		urls;
    char	*url_cp;
#    define	 DCC_FUZ2_URL_MAX   (1+DCC_URL_MAX)
    char	url_buf[DCC_FUZ2_URL_MAX*2];
    DCC_URL_SKIP url;
    enum {
	DCC_FUZ2_ST_WORD = 0,		/* gathering word */
	DCC_FUZ2_ST_CREF,		/* get HTML character reference */
	DCC_FUZ2_ST_CREF_NUM,		/*	hex or decimal */
	DCC_FUZ2_ST_CREF_DEC,
	DCC_FUZ2_ST_CREF_HEX,
	DCC_FUZ2_ST_CREF_NAME,
	DCC_FUZ2_ST_START_TAG,		/* gathering start of HTML tag */
	DCC_FUZ2_ST_SKIP_TAG,		/* skipping HTML tag */
	DCC_FUZ2_ST_SKIP_COMMENT	/* skipping HTML comment */
    } state;
    MD5_CTX	md5;
} DCC_CTX_FUZ2;

/* The maximum length of a checksumed HELO value and the continuation string
 * need to be the same for all users of the local white-/blacklist */
#define DCC_HELO_MAX	(MAXHOSTNAMELEN+30)
#define	DCC_HELO_CONT	"..."

/* do not checksum more than this much of a header line */
#define DCC_HDR_CK_MAX	1024

/* substitute or locally configured checksums */
#define DCC_MAX_SUB_CKS 6

#define DCC_WSUMS (DCC_DIM_CKS+DCC_MAX_SUB_CKS)	/* max whitelist checksums */
typedef struct {
    DCC_GOT_SUM	sums[DCC_WSUMS];
    struct {				/*  quoted-printable state machine */
	int	    x;
	int	    y;
	u_char	    n;
	enum {
	    DCC_CK_QP_IDLE,		/* waiting for '=' */
	    DCC_CK_QP_EQ,		/* seen "=" */
	    DCC_CK_QP_1,		/* seen "=X" of "=XY" */
	    DCC_CK_QP_FAIL1,		/* have output '=' after bad =X */
	    DCC_CK_QP_FAIL2,		/* have output '=' after bad =XY */
	    DCC_CK_QP_FAIL3,		/* have output 'X' after bad =XY */
	} state;
    } qp;
    struct {				/* base64 state machine */
	u_int32_t   quantum;
	int	    quantum_cnt;
    } b64;
    enum {
	CK_MP_ST_PREAMBLE,		/* preamble before first boundary */
	CK_MP_ST_BND,			/* MIME boundary */
	CK_MP_ST_HDRS,			/* headers after a boundary */
	CK_MP_ST_TEXT,			/* body or entity */
	CK_MP_ST_EPILOGUE		/* text after last boundary */
    } mp_st;
    enum CK_MHDR_ST {			/* parse entity fields */
	CK_MHDR_ST_IDLE,
	CK_MHDR_ST_CE_CT,		/* matching "Content-T" */
	CK_MHDR_ST_CE,			/*	"ransfer-Encoding:" */
	CK_MHDR_ST_CE_WS,		/* white space after "encoding:" */
	CK_MHDR_ST_CT,			/*	"ype:" */
	CK_MHDR_ST_CT_WS,		/* white space after "type:" */
	CK_MHDR_ST_QP,			/*	"quoted-printable" */
	CK_MHDR_ST_B64,			/*	"base64" */
	CK_MHDR_ST_TEXT,		/*	"text" */
	CK_MHDR_ST_HTML,		/*	"/html" */
	CK_MHDR_ST_MULTIPART,		/*	"multipart" */
	CK_MHDR_ST_BND_BREAK_WS,	/* skip "/alternative" or similar */
	CK_MHDR_ST_BND_SPAN_WS,		/* skip whitespace before "boundary" */
	CK_MHDR_ST_BND,			/* match "boundary=" */
	CK_MHDR_ST_BND_VALUE		/* collecting boundary */
    } mhdr_st;
    u_char	mhdr_pos;
    u_char	mime_nest;		/* # of nested MIME entities */
    u_char	mime_bnd_matches;	/* # of active boundary matches */
    DCC_CK_BND	mime_bnd[3];
    enum {
	DCC_CK_CT_TEXT = 0,
	DCC_CK_CT_HTML,
	DCC_CK_CT_BINARY
    } mime_ct;
    enum {
	DCC_CK_CE_ASCII = 0,
	DCC_CK_CE_QP,
	DCC_CK_CE_B64
    } mime_ce;
    struct {
	u_int	    total;		/* non-whitespace text */
	u_char	    flen;		/* bytes of ">From" seen */
	MD5_CTX	    md5;
    } ctx_body;
    DCC_CTX_FUZ1 fuz1;
    DCC_CTX_FUZ2 fuz2;
    u_char	flags;
#    define	 DCC_CKS_HAVE_SUM   0x01    /* have at least one checksum */
#    define	 DCC_CKS_MIME_BOL   0x02    /* header decoder is at BOL */
#    define	 DCC_CKS_MIME_QUOTED 0x04
} DCC_GOT_CKS;

typedef DCC_TGTS DCC_CKS_WTGTS[DCC_WSUMS];


#define DCC_WHITE_SUFFIX ".dccw"

typedef u_int DCC_WHITE_INX;		/* 1-based index into hash table */

typedef char DCC_WHITE_MAGIC[128];

typedef struct {
    DCC_WHITE_INX fwd;
    DCC_TGTS	tgts;
    u_char	gen;
    u_char	pad[2];
    DCC_CK_TYPE type;
    DCC_SUM	sum;
} DCC_WHITE_ENTRY;

#define DCC_MAGIC_BINS 8209
typedef struct {
    DCC_WHITE_MAGIC magic;
    struct {
	DCC_WHITE_INX entries;		/* # of entries in the file */
	time_t	resolve;		/* when IP addresses last resolved */
#	 define	 DCC_WHITE_RESOLVE_BAD	    0   /* hash table broken */
#	 define	 DCC_WHITE_RESOLVE_NO_IP    1	/* never re-resolve */
	u_char	gen;			/* generation number */
	DCC_PATH white_incs[8];		/* included files */
    } hdr;
    DCC_WHITE_INX bins[DCC_MAGIC_BINS];	/* 1-based indeces or 0 for empty */
    DCC_WHITE_ENTRY tbl[1];
} DCC_WHITE_TBL;


/* everything there is to know about a currently active whitelist file */
typedef struct {
    DCC_WHITE_TBL *info;
    void    *mutex;
    DCC_WHITE_INX free;
    u_int   info_entries;		/* # of entries mapped window */
    u_int   info_size;			/* bytes in mapped window */
    time_t  ht_broken;			/* when to check sick file */
    time_t  stat_secs;			/* when to check ASCII files */
    HANDLE  ht_fd;
    FILE    *ascii_file;
    DCC_PATH ascii_nm;
    DCC_PATH ht_nm;
    u_char  per_user;			/* 1=hostnames allowed */
    u_char  info_dirty;			/* 1=need msync() */
    u_char  ht_checked;			/* 1=file has good magic */
    u_char  ht_locked;			/* 0=hash unlocked, 1=shared, 2=write */
} DCC_WF;

typedef enum {				/* greylist result */
    DCC_GREY_ASK_FAIL,			/* greylist server or other failure */
    DCC_GREY_OFF,			/* client whitelist or blacklist */
    DCC_GREY_ASK_EMBARGO_START,
    DCC_GREY_ASK_EMBARGO,
    DCC_GREY_ASK_EMBARGO_END,		/* first time server says ok */
    DCC_GREY_ASK_PASS,			/* greylist server says ok */
    DCC_GREY_ASK_WHITE,			/* greylist server says whitelisted */
} DCC_ASK_GREY_RESULT;


extern void dcc_wf_init(DCC_WF *, void *, u_char);
extern void dcc_wf_lock(DCC_WF *);
extern void dcc_wf_unlock(DCC_WF *);

extern void dcc_str2ck(DCC_SUM, const char *, u_int, const char *);
extern u_char dcc_get_cks(DCC_GOT_CKS *, DCC_CK_TYPES, const char *, u_char);
extern u_char dcc_ck_get_sub(DCC_GOT_CKS *, const char *, const char *);
extern u_char dcc_add_sub_hdr(DCC_EMSG, const char *);
extern void dcc_ck_ipv6(DCC_SUM, const struct in6_addr *);
extern void dcc_get_ipv6_ck(DCC_GOT_CKS *, const struct in6_addr *);
extern u_char dcc_get_str_ip_ck(DCC_GOT_CKS *, const char *, struct in6_addr *);
extern u_char dcc_str2ck_ip(DCC_SUM, const char *, struct in6_addr *);
extern void dcc_print_cks(void *, const DCC_GOT_CKS *, DCC_CKS_WTGTS,
			  void (out)(void *, const void *, u_int));
extern void dcc_print_grey(void *, DCC_ASK_GREY_RESULT, u_char *,
			   const char *, const DCC_SUM,
			   void (out)(void *, const void *, u_int));
extern u_char dcc_grey_spam(DCC_EMSG, DCC_CLNT_CTXT *, const DCC_SUM);

extern void dcc_ck_body_init(DCC_GOT_CKS *);
extern void dcc_ck_mime_hdr(DCC_GOT_CKS *, const char *, const char *);
extern u_char parse_mime_hdr(DCC_GOT_CKS *, const char *, u_int, u_char);
extern void dcc_ck_body(DCC_GOT_CKS *, const void *, u_int);
extern void dcc_ck_body_fin(DCC_GOT_CKS *);

extern u_char dcc_get_white(DCC_EMSG, DCC_WHITE_INX);

typedef int (*DCC_PARSE_CK_FNC)(DCC_EMSG, DCC_WF *,
				const char *, int,  /* fnm and lineno */
				DCC_CK_TYPES,	/* type of checksum */
				DCC_SUM,    /* computed checksum */
				DCC_TGTS);  /* "OK2" etc */
typedef u_char (*DCC_PARSE_CK_LOCK)(DCC_EMSG, DCC_WF*);
extern int dcc_parse_ck(DCC_EMSG, DCC_WF *wf, const char *, int,
			const char *, const char *, DCC_TGTS,
			DCC_PARSE_CK_FNC, DCC_PARSE_CK_LOCK);
extern int dcc_parse_hex_ck(DCC_EMSG, DCC_WF *wf,
			    const char *, int, const char *,
			    const char *, DCC_TGTS, DCC_PARSE_CK_FNC);
extern u_char dcc_parse_whitefile(DCC_EMSG, DCC_WF *, const DCC_PATH, FILE *,
				  DCC_WHITE_TBL **,
				  DCC_PARSE_CK_FNC, DCC_PARSE_CK_LOCK);

u_char dcc_white_unlock(DCC_EMSG, DCC_WF *, u_char);
typedef enum {
    DCC_WHITE_ERROR,
    DCC_WHITE_HALF_LISTED,
    DCC_WHITE_LISTED,
    DCC_WHITE_UNLISTED,
    DCC_WHITE_BLACK
} DCC_WHITE_RESULT;
extern u_char dcc_white_rdy(DCC_EMSG, DCC_WF *, const char *,
			    u_char, DCC_WHITE_RESULT *);
extern DCC_WHITE_RESULT dcc_white_sum(DCC_EMSG, DCC_WF *,
				      DCC_CK_TYPES, const DCC_SUM, u_char);
extern DCC_WHITE_RESULT dcc_white_cks(DCC_EMSG, DCC_WF *, const DCC_GOT_CKS *,
				      DCC_CKS_WTGTS, u_char, u_char);

extern int dcc_ask(DCC_EMSG, DCC_CLNT_CTXT *, DCC_HEADER_BUF *,
		   DCC_GOT_CKS *, u_char *, DCC_TGTS);
extern u_char dcc_white_ask(DCC_EMSG, DCC_WF *, DCC_CLNT_CTXT *,
			    DCC_HEADER_BUF *, u_char *, const char *,
			    DCC_GOT_CKS *, DCC_CKS_WTGTS, DCC_TGTS);
extern void dcc_init_tholds(void);
extern void dcc_parse_honor(const char *);
extern u_char dcc_parse_tholds(char, const char *);
extern void dcc_honor_log_cnts(u_char *, const DCC_GOT_CKS *, DCC_TGTS);
#   define  DCC_HONOR_SRVR_OK2	    0x01    /* have honored DCC_TGTS_OK2 */
#   define  DCC_HONOR_SRVR_NOTSPAM  0x02    /* known not spam by DCC */
#   define  DCC_HONOR_SRVR_ISSPAM   0x04    /* spam by DCC server */
#   define  DCC_HONOR_LOCAL_ISSPAM  0x08    /* blacklisted */
#   define  DCC_HONOR_LOGIT	    0x10    /* need to log this message */
extern DCC_ASK_GREY_RESULT dcc_ask_grey(DCC_EMSG, DCC_CLNT_CTXT *, DCC_OPS,
					DCC_SUM, const DCC_GOT_CKS *,
					const DCC_SUM, u_char *);

#endif /* DCC_CK_H */
