/* Distributed Checksum Clearinghouse internal client definitions
 *
 * Copyright (c) 2003 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.7-1.47 $Revision$
 */

#ifndef DCC_CLNT_H
#define DCC_CLNT_H

#include "dcc_defs.h"


#define DCC_MAX_XMITS	    4		/* log(DCC_MAX_RTT/DCC_INIT_RTT) */
#define	DCC_RTT_BAD	    (DCC_MAX_RTT*2+1)
#define DCC_RTT_ADJ_MAX	    (DCC_MAX_RTT/2)

#define	DCC_RE_RESOLVE	    (60*60*2)	/* check new DNS A RR's every 2 hours */

#define DCC_MAX_SRVR_NMS    8		/* server hostnames */
#define DCC_MAX_SRVR_ADDRS  12		/* server IP addresses */

extern u_char dcc_clnt_debug;
extern u_int dcc_min_delay_us;		/* override minimum RTT */

extern char dcc_clnt_hostname[MAXHOSTNAMELEN+1];

/* information kept by a client about a server */
typedef struct {
    int		rtt_adj;
    DCC_CLNT_ID	clnt_id;		/* our ID sent to the server */
    u_int16_t	port;			/* network byte order port # */
    u_char	defined;		/* name resolves into an IP address */
    char	hostname[MAXHOSTNAMELEN+1];
    DCC_PASSWD	passwd;
} DCC_SRVR_NM;

/* the port number must be installed in this before use */
#define DCC_SRVR_NM_DEF {0, DCC_ID_ANON, 0, 0, {"127.0.0.1"}, {""}}

typedef struct {			/* a single server address */
    DCC_SOCKU	su;
    time_t	rtt_updated;		/* when RTT last updated */
    int		rtt;			/* round trip time in microseconds */
    int		srvr_wait;		/* microseconds server penalizes us */
    u_int32_t	resp_mem;		/* 1 bit per recent xmit */
#    define	 DCC_TOTAL_XMITS_MAX (sizeof(u_int32_t)*8)
    u_int16_t	nm_inx;			/* -1, index in DCC_CLNT_INFO.nms */
    DCC_SRVR_ID	srvr_id;		/* the server's claimed ID */
    u_char	total_xmits;
    u_char	total_resps;
    u_char	srvr_pkt_vers;
    u_char	unused[1];
    DCC_BRAND	brand;			/* the server's announced brand name */
} DCC_SRVR_ADDR;

typedef struct {
    time_t	resolve;		/* when names need to be resolved */
    time_t	measure;		/* when RTT measurements need */
    int		gen;
    int		base_rtt;
    int		thold_rtt;		/* threshold for switching */
    int		avg_thold_rtt;		/* long term threshold */
    int		num_addrs;		/* total resolved addresses */
    time_t	fail_time;		/* DCC broken until then */
    u_char	fail_exp;		/* backoff */
#    define	 DCC_INIT_FAIL_SECS  4
#    define	 DCC_MAX_FAIL_EXP    6
#    define	 DCC_MAX_FAIL_SECS   (DCC_INIT_FAIL_SECS<<DCC_MAX_FAIL_EXP)
    u_char	act_inx;		/* -1 or active index in srvrs.addrs */
    DCC_SRVR_NM	nms[DCC_MAX_SRVR_NMS];
    DCC_SRVR_ADDR addrs[DCC_MAX_SRVR_ADDRS];
} DCC_SRVR_CLASS;

/* Information about DCC servers shared among clients on a host
 *	This is the shape of the mmap()'ed file. */
typedef struct {
#    define	 DCC_MAP_INFO_VERSION "DCC client mapped data version #7"
    char	version[36];
    u_int	residue;		/* for picking addresses */
#    define	 DCC_SRVRS_MOD	59
    u_char	flags;
#    define	 DCC_CLNT_INFO_FG_IPV6	0x01
#    define	 DCC_CLNT_INFO_FG_SOCKS	0x02
    DCC_HDR     proto_hdr;		/* prototype DCC request header */
    DCC_SRVR_CLASS dcc;			/* normal DCC services */
    DCC_SRVR_CLASS grey;		/* grey listing */
}  DCC_CLNT_INFO;

#define	DCC_MAP_INFO_VERSION_6 "DCC client mapped data version #6"
#ifdef DCC_MAP_INFO_VERSION_6
#define DCC_V6_MAX_SRVR_NMS 8
typedef struct {
    int		num_addrs;		/* total resolved addresses */
    u_int	residue;		/* for picking addresses */
    int		gen;
    DCC_SRVR_ADDR addrs[12];
} DCC_V6_SRVRS;

typedef struct {
    char	version[36];
    time_t	resolve;		/* when names need to be resolved */
    time_t	measure;		/* when RTT measurements need */
    time_t	srvr_fail_time;		/* servers broken until then */
    int		base_rtt;
    int		thold_rtt;		/* threshold for switching */
    int		avg_thold_rtt;		/* long term threshold */
    u_char	unused[8];
    u_char	srvr_fail_exp;		/* backoff */
    u_char	act_inx;		/* -1 or active index in srvrs.addrs */
    u_char	total_addrs;
    u_char	flags;
    DCC_HDR     proto_hdr;		/* prototype DCC request header */
    DCC_SRVR_NM	nms[DCC_V6_MAX_SRVR_NMS];
    DCC_V6_SRVRS srvrs;
} DCC_V6_CLNT_INFO;

typedef struct {
    DCC_CLNT_ID	clnt_id;
    u_int16_t	port;
    char	hostname[MAXHOSTNAMELEN+1];
    DCC_PASSWD	passwd;
    signed char	rtt_adj;
} DCC_V5_SRVR_NM;
typedef struct {
    DCC_SOCKU	su;
    u_int16_t	anum;
    DCC_SRVR_ID	srvr_id;
    int		rtt;
    int		srvr_wait;
    u_int32_t	resp_mem;
    u_char	cur_xmits;
    u_char	cur_resps;
    u_char	total_xmits;
    u_char	total_resps;
    DCC_BRAND	brand;
    u_char	srvr_pkt_vers;
    u_char	unused[3];
} DCC_V5_SRVR_ADDR;

typedef struct {
    int		num_addrs;		/* # of resolved addresses */
    int		gen;
    DCC_V5_SRVR_ADDR addrs[DCC_V6_MAX_SRVR_NMS*2];
} DCC_V5_SRVRS;

typedef struct {
#    define	 DCC_MAP_INFO_VERSION_5 "DCC client mapped data version #5"
    char	version[36];
    time_t	resolve;		/* when names need to be resolved */
    int		min_delay;		/* microsecond sum of RTT & srvr_wait */
    u_int16_t	act_inx;		/* -1 or active index in srvrs.addrs */
    u_char	unused[13];
    u_char	total_addrs;
    u_char	working_addrs;
    u_char	flags;
    DCC_HDR     proto_hdr;		/* prototype DCC request header */
    DCC_V5_SRVR_NM nms[DCC_V6_MAX_SRVR_NMS];
    DCC_V5_SRVRS srvrs;
} DCC_V5_CLNT_INFO;
#endif /* DCC_MAP_INFO_VERSION_6 */

#define DCC_IS_GREY(class)	((class) == &dcc_clnt_info->grey)
#define	DCC_GREY2PORT(mode)	htons((mode)? DCC_GREY_PORT : DCC_SRVR_PORT)
#define DCC_CLASS2PORT(class)	DCC_GREY2PORT(DCC_IS_GREY(class))

extern DCC_CLNT_INFO *dcc_clnt_info;    /* memory mapped shared data */
extern u_char dcc_all_srvrs;	/* try to contact all servers */


typedef struct {			/* record of 1 transmission */
    DCC_OP_NUMS	op_nums;
    time_t	sent_us;
    DCC_OPS	op;
    DCC_CLNT_ID	id;
    int		addrs_gen;
    DCC_SRVR_ADDR *addr;
    DCC_PASSWD  passwd;
} DCC_XLOG_ENTRY;

typedef struct {
    u_char	outstanding;		/* unheard responses */
    u_char	working_addrs;
    struct {
	u_char	    xmits;
	u_char	    resps;
    } cur[DCC_MAX_SRVR_ADDRS];
    DCC_SRVR_CLASS *class;
    DCC_XLOG_ENTRY *base;		/* start of array */
    DCC_XLOG_ENTRY *next;		/* next entry to use */
    DCC_XLOG_ENTRY *last;		/* last entry in array */
} DCC_XLOG;

/* invalid timer */
#define DCC_OP_NUMS_NULL ((u_int32_t)-1)


typedef struct dcc_clnt_ctxt {
    struct dcc_clnt_ctxt *fwd;
    struct timeval now, start;
    int		now_us;
    SOCKET	soc;
    DCC_SOCKU	conn_su;		/* sockect connected to this address */
    DCC_XLOG	xlog;
    DCC_XLOG_ENTRY xlog_entries[DCC_MAX_XMITS];
    u_char	use_ipv6;
} DCC_CLNT_CTXT;

#define DCC_INFO_USE_IPV4  "IPv6 off"
#define DCC_INFO_USE_IPV6  "IPv6 on"
#define DCC_INFO_USE_SOCKS  "use SOCKS"


/* many POSIX thread implementations have unexpected side effects on
 * ordinary system calls, so do allow the calling application
 * to not use threads by having both threaded and unthreaded
 * DCC client interfaces */

extern u_char dcc_host_locked;
extern void dcc_ctxts_lock(void);
extern void dcc_ctxts_unlock(void);
extern void dcc_syslog_lock(void);
extern void dcc_syslog_unlock(void);
extern void dcc_host_lock(void);
extern void dcc_host_unlock(void);
void dcc_freehostent(void);
extern u_char dcc_resolve_mutex_lock(u_char);
extern void dcc_resolve_mutex_unlock(void);
extern void dcc_mutex_init(void *, const char *);
extern u_char dcc_clnt_wake_resolve(void);
extern void dcc_clnt_stop_resolve(void);
extern void dcc_clnt_unthread_init(void);
extern void dcc_clnt_thread_init(u_char);


extern FILE *dcc_open_srvr_nm(DCC_EMSG, const char *);
extern const char *dcc_parse_nm_port(DCC_EMSG, const char *, u_int,
				     char *, u_int, u_int16_t *, char *, u_int,
				     const char *, int);
extern int dcc_parse_srvr_nm(DCC_EMSG, DCC_SRVR_NM *, u_char *,
			     const char *, const char *, int);
extern const char *dcc_srvr_nm(void);
extern DCC_CLNT_CTXT *dcc_alloc_ctxt(void);
extern u_char dcc_rel_ctxt(DCC_EMSG, DCC_CLNT_CTXT *);
extern u_char dcc_info_unlock(DCC_EMSG);
extern u_char dcc_info_lock(DCC_EMSG);
extern u_char dcc_resolve_lock(DCC_EMSG);
extern u_char dcc_resolve_unlock(DCC_EMSG);
extern void dcc_force_measure_rtt(DCC_SRVR_CLASS *, u_char);
extern void dcc_map_changed(void);
extern u_char dcc_create_map(DCC_EMSG, const char *, int *,
			     const DCC_SRVR_NM *, int,
			     const DCC_SRVR_NM *, int, u_char);
extern u_char dcc_unmap_info(DCC_EMSG);
extern u_char dcc_map_info(DCC_EMSG, const char *, HANDLE);
extern u_char dcc_map_lock_info(DCC_EMSG, const char *, HANDLE);
extern u_char dcc_map_lock_tmp_info(DCC_EMSG, const DCC_SRVR_NM *, u_char);
extern u_char dcc_clnt_rdy(DCC_EMSG, DCC_CLNT_CTXT *, u_char, u_char, u_char);
extern DCC_CLNT_CTXT *dcc_tmp_clnt_init(DCC_EMSG, DCC_CLNT_CTXT *,
					const DCC_SRVR_NM *, u_char, u_char);
extern DCC_CLNT_CTXT *dcc_clnt_init(DCC_EMSG, DCC_CLNT_CTXT *, const char *,
				    u_char, u_char, u_char);
extern u_char dcc_clnt_op(DCC_EMSG, DCC_CLNT_CTXT *, u_char, int *,
			  DCC_HDR *, int, DCC_OPS, DCC_HDR *, int, DCC_SOCKU *);
extern DCC_OPS dcc_aop(DCC_EMSG, DCC_CLNT_CTXT *, u_char, int,
		       DCC_AOPS, u_int32_t, u_char, u_char, u_char,
		       DCC_ADMN_RESP_VAL *, int *, DCC_SOCKU *);
extern void dcc_print_info(const char *, const DCC_CLNT_INFO *,
			   u_char, u_char, u_char);

extern int dcc_xhdr_start(char *, int);
extern void dcc_header_init(DCC_HEADER_BUF *);
extern void dcc_add_header(DCC_HEADER_BUF *, const char *,  ...) PATTRIB(2,3);
extern void dcc_add_header_ck(DCC_HEADER_BUF *, DCC_CK_TYPES, DCC_TGTS);


#endif /* DCC_CLNT_H */
