/* include/dcc_config.h.  Generated automatically by configure.  */
/* Distributed Checksum Clearinghouse
 *
 * autoconf configuration settings
 *
 * Copyright (c) 2003 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.7-1.38 $Revision$
 */

#ifndef DCC_CONFIG_H
#define DCC_CONFIG_H

#define UNIX

#define DCC_HOMEDIR	"/var/lib/dcc"
#define DCC_LIBEXECDIR	"/usr/sbin"

/* deal with systems such as Solaris that do not have a __P() macro
 *	assume they understand prototypes */
#define HAVE_SYS_CDEFS_H 1
#if !defined(HAVE_SYS_CDEFS_H) && !defined(__P)
#define __P(protos) protos
#endif

#ifdef _AIX41
#include <sys/select.h>
typedef unsigned long long int uint64_t;
typedef unsigned int uint32_t;
typedef int int32_t;
typedef unsigned short uint16_t;
/* AIX is missing some prototypes or has them #ifdef'ed with strange switches */
extern void openlog(const char *, int, int);
extern int snprintf(char *, int, const char *, ...);
extern int vsnprintf(char *, int, const char *, const char *, ...);
extern int seteuid(uid_t);
extern int flock(int, int);
#define	AF_LOCAL AF_UNIX
#endif /* _AIX41 */

/* some systems have uint32_t, others have u_int32_t, and some have both */
#define DCC_HAVE_U_INT32_T 1
#if !defined(DCC_HAVE_U_INT32_T)
#define u_int32_t uint32_t
#define u_int16_t uint16_t
#endif
/* and then there is u_*int64_t */
#define DCC_HAVE_U_INT64_T 1
#if !defined(DCC_HAVE_U_INT64_T)
#define u_int64_t uint64_t
#endif

/* build the server database for more than 16 million hashes */
/* #undef DCC_BIG_DB */

/* build the client library to work around Solaris broken fcntl() NFS locking */
/* #undef DCC_BAD_FCNTL_LOCKS */

/* use poll() instead of select() on Solaris because socket() can yield
 * file descripters larger than FD_SETSIZE */
/* #undef USE_POLL */


#ifdef __hpux
#define seteuid(euid) setresuid(-1,euid,-1)
#endif /* __hpux */

/* Define if you can safely include both <sys/time.h> and <time.h>.  */
#define TIME_WITH_SYS_TIME 1
#if TIME_WITH_SYS_TIME
# include <sys/time.h>
# include <time.h>
#else
# if HAVE_SYS_TIME_H
#  include <sys/time.h>
# else
#  include <time.h>
# endif
#endif


#define HAVE_GCC_ATTRIBUTES 1
#ifdef HAVE_GCC_ATTRIBUTES
#define UATTRIB __attribute__((unused))
#define PATTRIB(f,l) __attribute__((format (printf,f,l)))
#define NRATTRIB __attribute((__noreturn__))
#else
#define UATTRIB
#define PATTRIB(f,l)
#define NRATTRIB
#endif

#define HAVE_GCC_INLINE 1
#ifndef HAVE_GCC_INLINE
#define inline
#endif


/* fill holes in the target */
#define HAVE_DAEMON 1
#ifndef HAVE_DAEMON
#define daemon dcc_daemon
extern int daemon(int, int);
#endif
#define HAVE_VSYSLOG 1
#define HAVE_HSTRERROR 1
#define HAVE_INET_NTOP 1
#define HAVE_GETHOSTID 1


/* #undef NEED_STRINGS_H */
#ifdef NEED_STRINGS_H
#include <strings.h>
#endif

/* have getrlimit(RLIMIT_NOFILE) and getrlimit(RLIMIT_NOFILE) */
#define HAVE_RLIMIT_NOFILE 1

/* A way to get the size of physical memory
 *  Linux and Solaris have sysconf(_SC_PHYS_PAGES)
 *  BSD systems have sysctl(HW_PHYSMEM) */
/* #undef HAVE_PHYSMEM_TOTAL */
#define HAVE__SC_PHYS_PAGES 1
/* #undef HAVE_HW_PHYSMEM */

/* 0 or minimum size of server database buffer */
#define DCC_DB_MEMORY 33554432


/* 4.4BSD sockets */
/* #undef DCC_HAVE_SA_LEN */
#ifdef DCC_HAVE_SA_LEN
#define DCC_SU_LEN(s) ((s)->sa.sa_len)
#else
#define DCC_SU_LEN(s) (((s)->sa.sa_family == AF_INET)	\
		       ? sizeof((s)->ipv4) : sizeof((s)->ipv6))
#endif
#define DCC_HAVE_SOCKLEN_T 1
#ifdef DCC_HAVE_SOCKLEN_T
#define DCC_SOCKLEN_T socklen_t
#else
#define DCC_SOCKLEN_T int
#endif
#define DCC_HAVE_AF_LOCAL 1
#if !defined(DCC_HAVE_AF_LOCAL) && !defined(AF_LOCAL)
#define AF_LOCAL AF_UNIX
#endif

#define NO_IPV6 "no getipnodebyname()"
/* #undef NO_AF_INET6 */
#define CONF_S6_ADDR32 __u6_addr.__u6_addr32

/* length of address to disconnect a UDP socket */
#define DCC_UDP_DISCON sizeof(ctxt->conn_su.sa)

/* use getifaddrs() to get list of interface addresses */
#define HAVE_GETIFADDRS 1
/* #undef USE_DCC_GETIFADDRS */
/* Some systems have getifaddrs() but not freeifaddrs() */
#define HAVE_FREEIFADDRS 1

/* Solaris does not have paths.h */
#define HAVE_PATHS_H 1
#define DCC_RUNDIR "/var/run/dcc"

/* Some systems have their own MD5 libraries */
/* #undef HAVE_MD5 */

/* HP_UX has sys/pthread.h instead of pthread.h */
#define HAVE_PTHREAD_H 1

/* Cygwin does not have arpa/nameser.h and resolv.h */
#define HAVE_RESOLV_H 1
#define HAVE_ARPA_NAMESER_H 1

/* Windows systems lack UNIX permission bits */
#define HAVE_PRIVATE_FILES 1

/* files with 64-bit offsets */
/* #undef HAVE_BIG_FILES */

/* __progname defined by crt0 and so a reasonable default for syslog */
#define HAVE___PROGNAME 1

/* very old BSD/OS has only 2 parameters for msync()
 * and newer versions ignore the third parameter */
/* #undef HAVE_OLD_MSYNC */
#ifdef HAVE_OLD_MSYNC
#define MSYNC(addr,len,flags) msync((void *)(addr),(len))
#else
#define MSYNC(addr,len,flags) msync((void *)(addr),(len),(flags))
#endif

/* use SOCKS */
/* #undef HAVE_LIBSOCKS */


#endif /* DCC_CONFIG_H */
