/* Distributed Checksum Clearinghouse
 *
 * common definitions internal to client libraries and servers
 *
 * Copyright (c) 2003 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.7-1.137 $Revision$
 */

#ifndef DCC_DEFS_H
#define DCC_DEFS_H

#define RHYOLITE_VERSION "1.2.7"
#ifdef VERSION
#define DCC_VERSION RHYOLITE_VERSION"-"VERSION
#else
#define DCC_VERSION RHYOLITE_VERSION
#endif

#include "dcc_config.h"

#include <stdarg.h>
/* work on WIN32 and any reasonable UNIX platform */
#ifdef UNIX
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#undef EX_OK				/* IRIX defines EX_ok in unistd.h */
#include <sysexits.h>
#include <stdio.h>			/* for FreeBSD */
#include <limits.h>			/* for FreeBSD */
#include <netdb.h>
#include <netinet/in.h>
#include <sys/param.h>			/* for FreeBSD */
#include <sys/socket.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/mman.h>

#else /* !UNIX */
#define STRICT
#define _WINDOWS
#include <limits.h>
#include <winsock2.h>
#include <ws2tcpip.h>
#include <windowsx.h>
#include <sys/types.h>
#endif /* UNIX */

#include "dcc_proto.h"

#ifdef UNIX				/* deal with ancient UNIX */
#ifndef STDOUT_FILENO
#define STDOUT_FILENO 1
#endif
#ifndef STDERR_FILENO
#define STDERR_FILENO 2
#endif

#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

#ifndef FD_CLOEXEC
#define FD_CLOEXEC 1
#endif

typedef int SOCKET;
typedef int HANDLE;
#define INVALID_HANDLE_VALUE (-1)
#define INVALID_SOCKET (-1)
#define SOCKET_ERROR (-1)
#define closesocket close
#define ioctlsocket ioctl
#define CloseHandle close
#endif /* !UNIX */


/* define in6_addr here after u_int32_t has been seen in sys/types.h */
#ifdef NO_IPV6
struct dcc_in6_addr {			/*  IPv6 address. */
    u_int32_t dcc_s6_addr32[4];
};
#undef in6_addr
#define in6_addr dcc_in6_addr		/* defend against lurking definitions */
#undef s6_addr32
#define s6_addr32 dcc_s6_addr32

struct dcc_sockaddr_in6 {
    u_char	sin6_len;
    u_char	sin6_family;
    u_int16_t	sin6_port;
    u_int32_t	sin6_flowinfo;
    struct dcc_in6_addr sin6_addr;
};
#define sockaddr_in6 dcc_sockaddr_in6	/* defend against lurking definitions */
#endif

/* puzzle out something for s6_addr32 */
#if !defined(s6_addr32) && defined(CONF_S6_ADDR32)
#define s6_addr32 CONF_S6_ADDR32
#endif

#ifdef NO_AF_INET6
#define AF_INET6 24
#endif

#ifndef INET6_ADDRSTRLEN
#define INET6_ADDRSTRLEN 46
#endif
#ifndef INET_ADDRSTRLEN
#define INET_ADDRSTRLEN 16
#endif

#define MAXPORTNAMELEN	64

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffff
#endif

#define DCC_IP_EQ(a,b) ((a)->sa.sa_family == (b)->sa.sa_family		\
			&& (((a)->sa.sa_family == AF_INET)		\
			    ? ((a)->ipv4.sin_addr.s_addr		\
			       == (b)->ipv4.sin_addr.s_addr)		\
			    : !memcmp(&(a)->ipv6.sin6_addr,		\
				      &(b)->ipv6.sin6_addr,		\
				      sizeof((a)->ipv6.sin6_addr))))

#ifndef NTOHL
#define NTOHL(x) (x = ntohl(x))
#define NTOHS(x) (x = ntohs(x))
#define HTONL(x) (x = htonl(x))
#define HTONS(x) (x = htons(x))
#endif

#ifdef HAVE_BIG_FILES
#define OFF_HPAT    "%#llx"
#define OFF_DPAT    "%lld"
#else
#define OFF_HPAT    "%#lx"
#define OFF_DPAT    "%ld"
#endif

#define DIM(_a)	    ((int)(sizeof(_a) / sizeof((_a)[0])))
#define LAST(_a)    (&(_a)[DIM(_a)-1])
/* abbreviation to silence common compiler warnings */
#define ISZ(_s)	    ((int)sizeof(_s))

#define STRLIMCPY(d,lim,s) ((d)[(lim)-1] = '\0', strncpy(d,s,(lim)-1))
#define BUFCPY(d,s) STRLIMCPY(d,sizeof(d),s)

#define max(a,b) ((a) > (b) ? (a) : (b))
#define min(a,b) ((a) < (b) ? (a) : (b))

/* Is it time or has the local clock been changed?
 *	Is the clock after the target date or earlier than the original date? */
#define DCC_IS_TIME(now,tgt,lim) ((now) >= (tgt) || (now)+(lim) < (tgt))


#define DCC_WHITESPACE " \t\n\r"

/* ctype() is now a slow mess that does not give constant results on all
 * systems */
#define DCC_IS_WHITE(c) ((c) == ' ' || (c) == '\t' || (c) == '\r' || (c)== '\n')
#define DCC_IS_UPPER(c) ((c) >= 'A' && (c) <= 'Z')
#define DCC_IS_LOWER(c) ((c) >= 'a' && (c) <= 'z')
#define DCC_TO_LOWER(c) (DCC_IS_UPPER(c) ? ((c) + ('a'-'A')) : (c))
#define DCC_TO_UPPER(c) (DCC_IS_LOWER(c) ? ((c) - ('a'-'A')) : (c))

typedef union {
    struct sockaddr sa;
    struct sockaddr_in ipv4;
    struct sockaddr_in6 ipv6;
} DCC_SOCKU;

#ifdef HAVE_LIBSOCKS
extern int Rconnect(int, const struct sockaddr *, socklen_t);
extern ssize_t Rsend(int, const void *, size_t, int);
extern ssize_t Rsendto(int, const void *, size_t, int,
		       const struct sockaddr *, size_t);
extern ssize_t Rrecv(int, void *, size_t, int);
extern ssize_t Rrecvfrom(int, void *, size_t, int,
			 struct sockaddr *, socklen_t *);
extern struct hostent *Rgethostbyname(const char *);
#else
#define Rconnect    connect
#define Rsend	    send
#define Rsendto	    sendto
#define Rrecv	    recv
#define Rrecvfrom   recvfrom
#define Rgethostbyname gethostbyname
#endif

typedef char DCC_PASSWD[32];

#ifdef UNIX
typedef char DCC_PATH[PATH_MAX+1];
#define ERROR_STR() strerror(errno)
#else /* !UNIX */
typedef char DCC_PATH[MAX_PATH+1];
#endif /* !UNIX */

#define DCC_CONNECT_ERRORS(e) ((e) == ECONNREFUSED || (e) == EHOSTUNREACH   \
			       || (e) == EHOSTDOWN || (e) == ENETDOWN	    \
			       || (e) == ENETUNREACH)

typedef char DCC_EMSG[120];
extern int dcc_ex_code;			/* not thread safe */
extern void dcc_pemsg(int, DCC_EMSG, const char *, ...) PATTRIB(3,4);
extern void dcc_vpemsg(int, DCC_EMSG, const char *, va_list);

#define DCC_MAX_HDR_LINE    78		/* by RFC 2822 */
#define DCC_MAX_XHDR_LEN    240		/* largest possible X-DCC header */
typedef struct {
    u_int	start_len;		/* length of start up to ':' */
    u_int	used;			/* bytes of buffer used */
    int		col;			/* current column */
    char	buf[DCC_MAX_XHDR_LEN];
} DCC_HEADER_BUF;

#define DCC_MAP_NM_DEF	    "map"

extern void dcc_sign(const char *, int, void *, u_int);
extern u_char dcc_ck_signature(const char *, int, const void *, u_int);

extern DCC_PATH dcc_homedir;
extern DCC_PATH dcc_path_buf;
#define DCC_NM2PATH(nm) dcc_nm2path(dcc_path_buf, sizeof(dcc_path_buf), \
				    dcc_homedir, nm)
extern const char *dcc_nm2path(char *, u_int, const char *, const char *);
extern u_char dcc_cdhome(DCC_EMSG, const char *);

u_char dcc_get_priv_home(const char *);
extern void dcc_rel_priv(void);
extern void dcc_init_priv(void);
extern u_char dcc_ck_private(DCC_EMSG, struct stat *, const char *, int);
extern int dcc_lock_open(DCC_EMSG, const char *, int, u_char, int, u_char *);
extern u_char dcc_unlock_fd(DCC_EMSG, int, int, const char *, const char *);
extern u_char dcc_shlock_fd(DCC_EMSG, int, int, const char *, const char *);
extern u_char dcc_exlock_fd(DCC_EMSG, int, int, const char *, const char *);
#define DCC_LOCK_ALL_FILE   (-1)

extern char dcc_have_logdir;
extern DCC_PATH dcc_logdir;
#define DCC_MKSTEMP_LEN 6		/* characters added by dcc_mkstemp() */
extern int dcc_mkstemp(char *nm, int nm_len,
		       const char *, const char *, int, int);
#define	DCC_TMP_LOG_PAT "/tmp."
#define	DCC_FIN_LOG_PAT "/msg."
extern int dcc_logdir_ck(DCC_EMSG, const char *, int *);
extern u_char dcc_log_init(DCC_EMSG, const char *);
extern int dcc_log_open(DCC_EMSG, char *, int);
extern u_char dcc_log_keep(DCC_EMSG, char *, int);
#define DCC_LOG_DATE_PAT "VERSION: 3\nDATE: %s"
#define DCC_LOG_DATE_FMT "%x %X %Z"
#define DCC_LOG_MSG_SEP "\n### end of message body ########################\n"
#define DCC_LOG_TRN_MSG0 "### log truncated ######################"
#define DCC_LOG_TRN_MSG "\n"DCC_LOG_TRN_MSG0"\n"
#define DCC_LOG_TRN_MSG_CR "\r\n"DCC_LOG_TRN_MSG0"\r\n"



extern int dcc_get_secs(char *, char **, int, int, int);
extern u_int dcc_get_port(DCC_EMSG, const char *, u_int, const char *, int);
#define DCC_GET_PORT_INVALID    0x10000
extern const struct hostent *dcc_get_host(const char *, u_char, int *);
extern const struct hostent *dcc_get_host_SOCKS(const char *, u_char, int *);
#ifndef HAVE_HSTRERROR
#define hstrerror dcc_hstrerror
extern const char *hstrerror(int);
#endif
extern DCC_SOCKU *dcc_mk_su(DCC_SOCKU *, int, const void *, u_short);
extern u_char dcc_str2ip(DCC_SOCKU *, const char *);
extern int dcc_str2cidr(DCC_EMSG, struct in6_addr *, struct in6_addr *,
			const char *, const char *, int);
extern u_char dcc_udp_bind(DCC_EMSG, int *, DCC_SOCKU *);
extern DCC_CLNT_ID dcc_get_id(DCC_EMSG, const char *, const char *, int);
extern const char *dcc_get_srvr_id(DCC_EMSG, DCC_SRVR_ID *,
				   const char *, const char *,
				   const char *, int);
extern const char *dcc_parse_word(DCC_EMSG, char *, int,
				  const char *, const char *,
				  const char *, int);
extern const char *fnm_lineno(const char *, int);

extern void dcc_toipv6(struct in6_addr *, const struct in_addr);

extern const char *dcc_su2str(const DCC_SOCKU *);
extern const char *dcc_su2str_opt(const DCC_SOCKU *, u_int16_t, char);
extern void dcc_su2name(char *, int, const DCC_SOCKU *);
#if !defined(HAVE_INET_NTOP) || defined(NO_IPV6)
#define DCC_INET_NTOP dcc_inet_ntop
extern const char *DCC_INET_NTOP(int, const void *, char *, size_t);
#else
#define DCC_INET_NTOP inet_ntop
#endif

extern int dcc_error_priority, dcc_trace_priority;
extern u_char dcc_parse_log_opt(const char *);
extern void dcc_syslog_init(const char *, const char *);
extern DCC_PATH dcc_progname;
extern void dcc_vfatal_msg(const char *, va_list);
extern void dcc_verror_msg(const char *, va_list);
extern void dcc_error_msg(const char *, ...) PATTRIB(1,2);
extern void dcc_vtrace_msg(const char *, va_list);
extern void dcc_trace_msg(const char *, ...) PATTRIB(1,2);
#ifndef HAVE_VSYSLOG
#define vsyslog dcc_vsyslog
extern void vsyslog(int, const char *, va_list);
#endif

extern void NRATTRIB dcc_logbad(int, const char *, ...) PATTRIB(2,3);

extern char *dcc_ck2str(char *, u_int, DCC_CK_TYPES, const DCC_SUM);
#define DCC_CK2STR_LEN (sizeof(DCC_SUM)*2+sizeof(DCC_SUM)/4+1)
extern const char *dcc_ck2str_err(DCC_CK_TYPES, const DCC_SUM);
extern char *dcc_cnt2str(char *, u_int, DCC_TGTS, u_char);
extern char *dcc_type2str(char *, u_int, DCC_CK_TYPES, const char *, u_char);
extern const char *dcc_type2str_err(DCC_CK_TYPES, const char *, u_char);
extern DCC_CK_TYPES dcc_str2type(const char *);
extern DCC_TGTS dcc_str2cnt(const char *);

extern const char *dcc_op2str(DCC_OPS, DCC_AOPS, u_int32_t);
extern const char *dcc_req_op2str(const DCC_ADMN_REQ *);
extern const char *dcc_hdr_op2str(const DCC_HDR *);


#endif /* DCC_DEFS_H */
