/* Distributed Checksum Clearinghouse X-DCC-Warning definition
 *
 * Copyright (c) 2003 by Rhyolite Software
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND RHYOLITE SOFTWARE DISCLAIMS ALL
 * WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL RHYOLITE SOFTWARE
 * BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 * ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 *
 * Rhyolite Software DCC 1.2.7-1.23 $Revision$
 */

#ifndef DCC_XHDR_H
#define DCC_XHDR_H

/* the DCC SMTP header is of the form
 *
 * X-DCC-name-Metrics: host_addr server-ID; [bulk] cknm1=cnt1 cknm2=cnt2 ...
 *
 * where
 *    chost		is the FQDN of the DCC client that added the header
 *    server-ID		is the ID of the DCC server providing the counts
 *    cknm1, cknm2, ...	are names of checksums
 *    cnt1, cnt2, ...	are counts or special counts
 */
#define DCC_XHDR_START	"X-DCC-"
#define DCC_XHDR_CK_PAT	"%s=%s"

#define DCC_XHDR_BULK	"bulk"

/* names of checksums */
#define DCC_XHDR_TYPE_IP	    "IP"	    /* binary source IP addr */
#define DCC_XHDR_TYPE_ENV_FROM	    "env_From"	    /* envelope Mail From */
#define DCC_XHDR_TYPE_FROM	    "From"	    /* header line */
#define DCC_XHDR_TYPE_SUB	    "substitute"    /* random header line */
#define DCC_XHDR_TYPE_MESSAGE_ID    "Message-ID"    /* header line */
#define DCC_XHDR_TYPE_RECEIVED	    "Received"	    /* last header line */
#define DCC_XHDR_TYPE_BODY	    "Body"	    /* body */
#define DCC_XHDR_TYPE_FUZ1	    "Fuz1"	    /* filtered body */
#define DCC_XHDR_TYPE_FUZ2	    "Fuz2"	    /*   "       "   */
#define DCC_XHDR_TYPE_FUZ3	    "Fuz3"	    /*   "       "   */
#define DCC_XHDR_TYPE_GREY_TRIPLE   "Grey3"	    /* greylist triple cksum */
#define DCC_XHDR_TYPE_SRVR_ID	    "server-ID"	    /* check dup. server-ID */
#define DCC_XHDR_TYPE_ENV_TO	    "env_To"	    /* envelope Rcpt To */
#define DCC_XHDR_TYPE_FLOD_PATH	    "flood path"
#define DCC_XHDR_MAX_TYPE_LEN  (ISZ("unknown xxx"))



/* strings used in X-DCC counts */
#define DCC_XHDR_TOO_MANY	"many"	    /* too many targets to number */
#define DCC_XHDR_GREY_OK	"pass"	    /* embargo has ended */
#define DCC_XHDR_OK		"ok"	    /* certified not spam */
#define DCC_XHDR_OK2		"ok2"	    /* half certified not spam */
#define DCC_XHDR_DEL		"del-req"   /* delete request */
#define DCC_XHDR_MAX_TGTS_LEN	12

/* special server-IDs */
#define DCC_XHDR_ID_INVALID	"invalid"	/* DCC_ID_INVALID */
#define DCC_XHDR_ID_ANON	"anon"		/* DCC_ID_ANON */
#define DCC_XHDR_ID_WHITE	"whitelist"	/* DCC_ID_WHITE */
#define DCC_XHDR_ID_COMP	"compressed"	/* DCC_ID_COMP */
#define DCC_XHDR_ID_DELETED	"deleted"	/* DCC_ID_DELETED */

#endif /* DCC_XHDR_H	*/
